C**********************************************************************************************************
C
C				BAYESIAN INVERSION OF THE FREE CORE RESONANCE
C (REF. PAPER: FLORSCH AND HINDERER, 2000. BAYESIAN ESTIMATION OF THE FREE CORE NUTATION PARAMETERS FROM THE 
C ANALYSIS OF PRECISE TIDAL GRAVITY DATA, PEPI, 117, 21-35.)
C
C												Initial code by Nicolas Florsch
C												Revised by Severine Rosat, 2007
C	Main revisions:
C		* program rewritten in DOUBLE PRECISION instead of REAL*8;
C		* comments added;
C		* inputs and outputs.
C**********************************************************************************************************
C			MODIFICATIONS FROM THE INITIAL PAPER:
C	1/ THE IMAGINARY PART OF THE REFERENCE GRAVIMETRIC FACTOR IS MUCH SMALLER THAN THE ERRORS OF THE OCEAN
C	LOADING EFFECTS, SO IT IS PUT TO ZERO;
C	2/ THE REFERENCE GRAVIMETRIC FACTOR (DELTAREF) IS REAL AND FIXED. IT CAN BE THE MEAN VALUE OF THE PREDICTED 
C	GRAVIMETRIC FACTORS	FOR THE SYMMETRIC TIDAL WAVES O1 AND OO1;
C	3/ THE IMAGINARY PART OF THE RESONANCE STRENGTH IS PUT TO ZERO AS IT CANNOT BE DETERMINED (CF. PAPER);
C
C FINALLY THE 3 PARAMETERS INVERSED IN THIS VERSION ARE:
C AR (REAL PART OF THE RESONANCE STRENGTH), T (PERIOD) AND X = LOG(Q)
C**********************************************************************************************************
C
C	WARNING!!! A FEW POINTS MUST BE TREATED CAREFULLY WHEN USING THIS BAYESIAN METHOD
C	1/ ERRORS GIVEN BY ETERNA (OR OTHER SOFTWARE) TIDAL ANALYSIS ARE UNDERESTIMATED: THE ERRORS ON THE 
C	OCEANIC LOADING EFFECT MUST BE ADDED;
C	2/ PROBLEMS MAY OCCUR BECAUSE OF THE DOMAIN OF INVESTIGATION: THE INVESTIGATED DOMAIN MUST BE LIMITED
C	BECAUSE OF THE COMPUTATION LIMITS BUT NOT TOO MUCH AS IT MUST CONTAIN THE SOLUTION; IF NaN OR -Infinity
C	APPEAR IN THE SOLUTION, THE RANGE LIMITS OF THE PARAMETERS MUST BE CHANGED.
C	3/ THERE CAN BE SOME ERRORS IN THE PROGRAM WHEN THE EXPONENTIAL FUNCTIONS BECOME TOO SMALL. 
C	A FACTOR 10**20 IS MULTIPLIED TO THE EXPONENTIALS TO AVOID SUCH A PROBLEM.
C	4/ WHEN COMPUTING THE MARGINAL LAWS, WE IMPLICITLY LIMIT THE RANGE DOMAIN OF THE PARAMETER BY INTEGRATING
C	ON THIS DOMAIN;
C	5/ MULTI-STATION APPLICATION: THE MARGINAL LAW OF A PRODUCT IS NOT THE PRODUCT OF THE MARGINAL LAWS;
C	6/ MULTI-STATION APPLICATION: THE RESONANCE STRENGTH IS SUPPOSED TO BE THE SAME FOR ALL THE STATIONS.
C
C**********************************************************************************************************
C	PROGRAM INPUTS:
C	* INIFILE (e.g. "BAYES.INI")	File containing the options used in the program. Example of INI file:
C	|INPUT FOR THE BAYESIAN INVERSION ROUTINE					
C	|NUMBER OF STATIONS											
C	|2															
C	|FILE NAMES CONTAINING THE OBSERVED DELTA FACTORS						
C	|FILE1.DAT														
C	|FILE2.DAT															
C	|INTERVALS OF SEARCH FOR THE 3 PARAMETERS AR,T AND X		
C	|0.0005  0.0008												
C	|380 540													
C	|3 10														
C	|NUMBER OF POINTS FOR THE GRID (<= 201)						
C	|101	
C	* The files FILE1.DAT AND FILE2.DAT	contain the real and imaginary parts of the 
C	observed gravimetric factors corrected from the oceanic loading effect and their errors (so 4 columns) 
C	at the 1st and 2nd station.
C	 The last line of the file (line 10) corresponds to the reference gravimetric factor. 
C	By default we use 9 diurnal tidal waves:
C	Q1  O1  M1   P1   K1   PSI1   PHI1  J1   OO1
C	The files FILE1.DAT and FILE2.DAT can be created from the ETERNA analysis results using the routine delphi2drdi.f												
C
C	PROGRAM OUTPUTS:
C	* "GGP_3" 1-column file containing the full probability density function values;
C	* "X-T-.grd", "X-A-.grd" and "T-A-.grd" the joint marginal laws normalized by 100.
C**********************************************************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C NUMBER OF TIDAL WAVES IN THE DIURNAL GROUP USED FOR THE INVERSION
C BY DEFAULT:
C	Q1  O1  M1   P1   K1   PSI1   PHI1  J1   OO1
      PARAMETER (NF=9)
C MAXIMUM SIZE OF THE PARAMETER GRID NB*NB*NB <= NMAX*NMAX*NMAX
	PARAMETER (NMAX=201)
      COMMON /DELTAR/DELTAR(NF)
      COMMON /DELTAI/DELTAI(NF)
      COMMON /DELR/DELR(NF)
      COMMON /DELI/DELI(NF)
      COMMON /FREQ/FREQ(NF)
C THE TABLE TAB CONTAINS THE PROBABILITY DENSITY FUNCTION VALUES
C FOR THE THREE PARAMETERS
	DOUBLE PRECISION TAB(NMAX,NMAX,NMAX),XMAR(NMAX,NMAX)
C STATION CODE (e.g. ST for STRASBOURG)
	CHARACTER*2 CSTAT
C OUTPUT FILE NAMES
	CHARACTER*34 XTNAME3,XARNAME3,TARNAME3
C INPUT FILE NAMES
	CHARACTER*30 INIFILE
	CHARACTER*30 NAMES(30),NAME1
C FREQUENCY VALUES FOR THE 9 TIDAL WAVES
C UNIT: CYCLE PER DAY (CPD)
	DATA FREQ/0.893244D0,0.929536D0,0.966446D0,0.997262D0,1.002738D0
     &  ,1.005476D0,1.008214D0,1.039030D0,1.075940D0/
C READ INPUT FILE BAYES.INI
	WRITE(*,*) 'INI-FILE: '
	READ(5,'(A)') INIFILE
	OPEN(2,FILE=INIFILE,STATUS='OLD')
	READ(2,*)
	READ(2,*)
	READ(2,*) NBSTAT
	PRINT*,'NUMBER OF STATIONS CONSIDERED=',NBSTAT
	READ(2,*)
	DO I = 1,NBSTAT
	  READ(2,'(A)',ERR=29) NAMES(I)
	ENDDO
	READ(2,*)
	READ(2,*) ARMIN,ARMAX
	READ(2,*) TMIN,TMAX
	READ(2,*) XMIN,XMAX
	READ(2,*)
	READ(2,*) NB
	PRINT*,'GRID SIZE = ',NB
	GOTO 11
   29	PRINT*,'ERROR IN BAYES.INI: NUMBER OF STATIONS TOO LARGE'
   11	CONTINUE
	CLOSE(2)
C INITIALIZATION OF THE NORMALIZATION PARAMETER
	NORM = 0.D0
C CONVERT THE FREQUENCIES FROM CPD TO DEGREE PER HOUR      
	DO I=1,NF
	  FREQ(I)=FREQ(I)*15
	ENDDO
C INITIALIZATION OF THE TABLE CONTAINING THE PROBABILITY DENSITY FUNCTION VALUES
      DO I1=1,NB
	  DO I2=1,NB
	    DO I3=1,NB
	      TAB(I1,I2,I3)=1.D0
	    ENDDO
	  ENDDO
	ENDDO
C MULTI-STATION ANALYSIS	
C LOOP ON THE NBSTAT STATIONS
	DO ISTAT=1,NBSTAT
        NAME1=NAMES(ISTAT)
C READ THE ANALYZED DELTA VALUES CONTAINED IN THE FILE BAYES_'//CSTAT//'.DAT
C THE DELTA HAVE BEEN CORRECTED FROM THE OCEAN LOADING EFFECT.
C THERE ARE NF LINES FOR THE NF TIDAL WAVES CONSISTING OF 4 COLUMNS:
C REAL AND IMAGINARY PARTS OF THE GRAVIMETRIC FACTOR AND THEIR RESPECTIVE ERRORS
C NOTE: THE ERRORS SHOULD INCLUDE THE ETERNA ANALYSIS RESULTING ERROR+THE OCEANIC TIDAL CORRECTION ERRORS
        OPEN(1,FILE=NAME1)
        DO I=1,NF
          READ(1,*) DELTAR(I),DELTAI(I),DELR(I),DELI(I)
        ENDDO
C THE LAST LINE (NF+1) OF THE FILE CONTAINS THE REFERENCE GRAVIMETRIC FACTOR
	  READ(1,*)DELREF
	  WRITE(*,*) 'DELTA REFERENCE: ',DELREF
	  PAUSE

C DEFINITIONS OF THE INTERVALS SPANNED BY THE SEARCHED PARAMETERS
C AR SPANS FROM ARMIN TO ARMAX WITHIN NB POINTS;
C T SPANS FROM TMIN TO TMAX WITHIN NB POINTS;
C X SPANS FROM XMIN TO XMAX WITHIN NB POINS.
C NB IS THE NUMBER OF POINTS DEFINED AT THE BEGINNING AND NB-1 IS THE NUMBER OF INTERVALS
C!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
C IMPORTANT: IF ONE OF JOINT THE MARGINAL LAW BECOMES INFINITY, YOUR INTERVALS WERE BADLY DEFINED, SO
C ADJUST THEM!
        XNB=DBLE(NB-1)
C COMPUTATION OF THE STEP     
        DDAR=(ARMAX-ARMIN)/XNB
        DDT=(TMAX-TMIN)/XNB
        DDX=(XMAX-XMIN)/XNB

C IN THIS VERSION OF THE BAYESIAN INVERSION, THERE ARE ONLY 3 PARAMETERS ==> WE CAN COMPUTE THE WHOLE TABLE
        DO I1=1,NB
          PRINT*,I1,'/',NB
		X=XMIN+(I1-1)*DDX
		DO I2=1,NB
		  T=TMIN+(I2-1)*DDT
		  DO I3=1,NB
			AR=ARMIN+(I3-1)*DDAR
C COMPUTATION OF THE FUNCTION TAB WHERE NF IS THE NUMBER OF FREQUENCIES
C BY CALLING THE SUBROUTINE CALCUL
			TAB(I1,I2,I3)=CALCUL(AR,T,X,DELREF,NF)*TAB(I1,I2,I3)
		  ENDDO
		ENDDO
	  ENDDO
C END OF THE STATION LOOP
      ENDDO

C WRITE THE OUTPUT TABLE
      OPEN(13, FILE='GGP_3')
	DO I1=1,NB
	  PRINT*,I1,'/',NB
        DO I2=1,NB
		DO I3=1,NB
		  WRITE(13,*)TAB(I1,I2,I3)
		ENDDO
	  ENDDO
	ENDDO
	CLOSE(13)
C--------------------------------------------------------------------------------
C COMPUTATION OF THE 2D-MARGINAL LAWS (X,T),(X,A),(T,A)
C BY SUMMING ON THE INDICES OF TAB
C OUTPUT FILES
	XTNAME3='X-T-.grd'
	XARNAME3='X-A-.grd'
	TARNAME3='T-A-.grd'
C NOTE THAT AR IS MULITPLIED BY 10000 IN THESE FILES AS ITS VALUES ARE VERY SMALL
C----						X-T			-------------------------------------------
C FOR THE JOINT MARGINAL LAW OF X AND T, THE SUM IS PERFORMED ON THE 3RD INDICE
      DO I1=1,NB
	  X=XMIN+(I1-1)*DDX  
	  PRINT*,I1,'/',NB
        DO I2=1,NB
		XMAR(I1,I2)=0.D0
		T=TMIN+(I2-1)*DDT
		DO I3=1,NB
		  XMAR(I1,I2)=XMAR(I1,I2)+TAB(I1,I2,I3)
		ENDDO
	  ENDDO
	ENDDO
C NORMALIZATION TO 100 OF XMAR
      ZMIN=1.D300
      ZMAX=-1.D300
C FIND MIN AND MAX OF XMAR
      DO I=1,NB
	  DO J=1,NB
		IF(XMAR(I,J).LT.ZMIN)ZMIN=XMAR(I,J)
		IF(XMAR(I,J).GT.ZMAX)ZMAX=XMAR(I,J)
        ENDDO
      ENDDO
C NORMALIZATION
	DO I=1,NB
	  DO J=1,NB
		XMAR(I,J)=XMAR(I,J)/ZMAX*100
		IF(DABS(XMAR(I,J)).LT.1.D-50)XMAR(I,J)=0.D0
	  ENDDO
	ENDDO
C SAVE THE MARGINAL LAWS INTO THE FILE XTNAME3
      OPEN(9,FILE=XTNAME3,STATUS='UNKNOWN')
	WRITE(9,'(A4)')"DSAA"
      WRITE(9,*)NB,NB
      WRITE(9,*)XMIN,XMAX
      WRITE(9,*)TMIN,TMAX
      WRITE(9,*)0,100
      DO J=1,NB
        WRITE(9,'(201G15.3)')(XMAR(I,J),I=1,NB)
      ENDDO
	CLOSE(9)
C----						X-AR		-------------------------------------------
C FOR THE JOINT MARGINAL LAW OF X AND AR THE SUM IS PERFORMED ON THE 2ND INDICE
      DO I1=1,NB
	  X=XMIN+(I1-1)*DDX
	  PRINT*,I1,'/',NB
        DO I3=1,NB
		XMAR(I1,I3)=0.D0
		AR=ARMIN+(I3-1)*DDAR
		DO I2=1,NB
		  XMAR(I1,I3)=XMAR(I1,I3)+TAB(I1,I2,I3)
		ENDDO
	  ENDDO
	ENDDO
C NORMALIZATION TO 100 OF XMAR
      ZMIN=1.D300
      ZMAX=-1.D300
C FIND MIN AND MAX OF XMAR
      DO I=1,NB
	  DO J=1,NB
		IF(XMAR(I,J).LT.ZMIN)ZMIN=XMAR(I,J)
		IF(XMAR(I,J).GT.ZMAX)ZMAX=XMAR(I,J)
        ENDDO
      ENDDO
C NORMALIZATION
	DO I=1,NB
	  DO J=1,NB
		XMAR(I,J)=XMAR(I,J)/ZMAX*100
		IF(DABS(XMAR(I,J)).LT.1.D-50)XMAR(I,J)=0.D0
	  ENDDO
	ENDDO
C SAVE THE MARGINAL LAWS INTO THE FILE XARNAME3
      OPEN(7,FILE=XARNAME3,STATUS='UNKNOWN')
      WRITE(7,'(A4)')"DSAA"
      WRITE(7,*)NB,NB
      WRITE(7,*)XMIN,XMAX
      WRITE(7,*)ARMIN*10000,ARMAX*10000
      WRITE(7,*)0,100
      DO J=1,NB
        WRITE(7,'(201G15.3)')(XMAR(I,J),I=1,NB)
      ENDDO
	CLOSE(7)
C----						T-AR			-------------------------------------------
C FOR THE JOINT MARGINAL LAW OF T AND AR, THE SUM IS PERFORMED ON THE 1ST INDICE
      DO I2=1,NB
	  T=TMIN+(I2-1)*DDT
	  PRINT*,I2,'/',NB
        DO I3=1,NB
		XMAR(I2,I3)=0.D0
		AR=ARMIN+(I3-1)*DDAR
		DO I1=1,NB
		  XMAR(I2,I3)=XMAR(I2,I3)+TAB(I1,I2,I3)
		ENDDO
	  ENDDO
	ENDDO
C NORMALIZATION TO 100 OF XMAR
      ZMIN=1.D300
      ZMAX=-1.D300
C FIND MIN AND MAX OF XMAR
      DO I=1,NB
	  DO J=1,NB
		IF(XMAR(I,J).LT.ZMIN)ZMIN=XMAR(I,J)
		IF(XMAR(I,J).GT.ZMAX)ZMAX=XMAR(I,J)
        ENDDO
      ENDDO
C NORMALIZATION
	DO I=1,NB
	  DO J=1,NB
		XMAR(I,J)=XMAR(I,J)/ZMAX*100
		IF(DABS(XMAR(I,J)).LT.1.D-50)XMAR(I,J)=0.D0
	  ENDDO
	ENDDO
C SAVE THE MARGINAL LAWS INTO THE FILE TARNAME3
      OPEN(8,FILE=TARNAME3,STATUS='UNKNOWN')
      WRITE(8,'(A4)')"DSAA"
      WRITE(8,*)NB,NB
      WRITE(8,*)TMIN,TMAX
      WRITE(8,*)ARMIN*10000,ARMAX*10000
	WRITE(8,*)0,100
      DO J=1,NB
        WRITE(8,'(201G15.3)')(XMAR(I,J),I=1,NB)
      ENDDO
	CLOSE(8)
C DISPLAY SOME VALUES ON THE SCREEN	
	WRITE(*,*) ARMIN,ARMAX
	WRITE(*,*) TMIN,TMAX
	WRITE(*,*) XMIN,XMAX
      WRITE(*,*) 'DDT, DDX, DDAR',DDT,DDX,DDAR
	CONTINUE
	PAUSE
      END
C*******************************************************************************************
C				FUNCTION CALCUL
C COMPUTE THE RESONANCE MODEL AT EACH POINT OF THE NB X NB X NB GRID
C*******************************************************************************************
      FUNCTION CALCUL(AR,T,X,DELREF,N)
C N IS THE NUMBER OF FREQUENCIES < NF
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      PARAMETER (NF=20)
      DIMENSION DELTR(NF),DELTI(NF)
C THE COMMON VARIABLES CONTAIN THE OBSERVED DATA
C DELTAR AND DELTAI ARE THE DELTA FACTORS OF THE ANALYSES WITH DELR AND DELI THEIR ERRORS
C COEF IS THE COEFFICIENT TO CONVERT THE FCN FREQUENCY TO THE SIDEREAL PERIOD
C COEF=86164.D0/86400.D0/15.D0
	PARAMETER(COEF=0.066484568D0)
      COMMON /DELTAR/DELTAR(NF)
      COMMON /DELTAI/DELTAI(NF)
      COMMON /DELR/DELR(NF)
      COMMON /DELI/DELI(NF)
      COMMON /FREQ/FREQ(NF)
C RELATION BETWEEN THE REAL PULSATION SR OF THE FCN AND ITS SIDEREAL PERIOD T
      SR=(1.D0/T+1.D0)/COEF
      
C COMPUTATION OF THE THEORETICAL GRAVIMETRIC FACTORS DELTR AND DELTI
C PROV CORRESPONDS TO 1/Q WHERE Q IS THE QUALITY FACTOR
      DO I=1,N
        PROV=10**(-X)
        FF=FREQ(I)
        DEN=1.D0/((FF-SR)**2+(SR*PROV*0.5D0)**2)
        DELTR(I)=DELREF+AR*(FF-SR)*DEN
        DELTI(I)=+0.5D0*AR*SR*PROV*DEN
      ENDDO
C COMPUTATION OF THE LEAST SQUARE FUNCTION
      SUM=0.D0 
      DO I=1,N
        SUM=SUM+((DELTR(I)-DELTAR(I))/DELR(I))**2
        SUM=SUM+((DELTI(I)-DELTAI(I))/DELI(I))**2
      ENDDO
C MULTIPLY BY 1E20 TO AVOID AN EXPONENTIAL TOO SMALL
      CALCUL=DEXP(-0.5D0*SUM)*1.D20
	IF(CALCUL.LT.1.D-80)CALCUL=0.D0
      RETURN
	END